/* gInfo - Copyright (C) 2004 bakunin - Andrea Marchesini 
 *                                      <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <editor.h>

GtkWidget *highlight_normal;
GtkWidget *highlight_xml;
GtkWidget *notebook;

static GtkWidget *vpaned;

typedef struct
{
  GtkWidget *widget;
  char *label;
  GtkWidget *box;
} graphic_element;

void graphic_view_parser (GtkWidget *, gpointer);
void graphic_pop (GtkWidget *, gpointer);
GtkWidget *graphic_create_sep_window (graphic_element *);
int graphic_sep_window_close (GtkWidget *, GdkEvent *, gpointer);

int
graphic_create_window (void)
{
  GtkWidget *vbox;
  GtkWidget *box;
  GtkWidget *button;
  GtkWidget *image;
  GtkWidget *label;

  GtkWidget *menubar;
  GtkWidget *sep;

  GtkWidget *highlight;
  GtkWidget *highlight_menu;
  GSList *highlight_group = NULL;
  GtkWidget *file;
  GtkWidget *file_menu;
  GtkWidget *file_item;
  GtkWidget *edit;
  GtkWidget *edit_menu;
  GtkWidget *edit_item;
  GtkWidget *view;
  GtkWidget *view_menu;
  GtkWidget *view_item;
  GtkWidget *tools;
  GtkWidget *tools_menu;
  GtkWidget *tools_item;
  GtkWidget *help;
  GtkWidget *help_menu;
  GtkWidget *help_item;

  GtkWidget *editor;
  GtkWidget *viewer;
  GtkWidget *tips;

  GtkWidget *ibox;

  char buf[1024];
  graphic_element *re;

  GtkAccelGroup *accel_group;

  accel_group = gtk_accel_group_new ();

  snprintf (buf, 1024, "%s %s - XML Module Editor", NAME, VERSION);

  editorwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (editorwin), buf);
  gtk_widget_set_size_request (editorwin, gdk_screen_width () * 2 / 3,
			       gdk_screen_height () * 2 / 3);
  gtk_widget_realize (editorwin);

  g_signal_connect ((gpointer) editorwin, "delete_event",
		    G_CALLBACK (graphic_quit), NULL);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (editorwin), vbox);

  menubar = gtk_menu_bar_new ();
  gtk_widget_show (menubar);
  gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, TRUE, 0);

  file = gtk_menu_item_new_with_mnemonic ("_File");
  gtk_widget_show (file);
  gtk_container_add (GTK_CONTAINER (menubar), file);

  file_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (file), file_menu);

  file_item = gtk_image_menu_item_new_with_mnemonic ("_New");
  gtk_widget_show (file_item);
  gtk_container_add (GTK_CONTAINER (file_menu), file_item);

  gtk_widget_add_accelerator (file_item, "activate", accel_group,
			      GDK_n, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  g_signal_connect ((gpointer) file_item, "activate", G_CALLBACK (editor_new),
		    NULL);

  image = gtk_image_new_from_stock ("gtk-new", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (file_item), image);

  file_item = gtk_image_menu_item_new_with_mnemonic ("_Open...");
  gtk_widget_show (file_item);
  gtk_container_add (GTK_CONTAINER (file_menu), file_item);

  gtk_widget_add_accelerator (file_item, "activate", accel_group,
			      GDK_o, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  g_signal_connect ((gpointer) file_item, "activate",
		    G_CALLBACK (editor_open), NULL);

  image = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (file_item), image);

  sep = gtk_separator_menu_item_new ();
  gtk_widget_show (sep);
  gtk_container_add (GTK_CONTAINER (file_menu), sep);

  editor_save_item = gtk_image_menu_item_new_with_mnemonic ("_Save...");
  gtk_widget_show (editor_save_item);
  gtk_container_add (GTK_CONTAINER (file_menu), editor_save_item);

  gtk_widget_add_accelerator (editor_save_item, "activate", accel_group,
			      GDK_s, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  g_signal_connect ((gpointer) editor_save_item, "activate",
		    G_CALLBACK (editor_save), NULL);

  image = gtk_image_new_from_stock ("gtk-save", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (editor_save_item),
				 image);

  file_item = gtk_image_menu_item_new_with_mnemonic ("Save _As...");
  gtk_widget_show (file_item);
  gtk_container_add (GTK_CONTAINER (file_menu), file_item);

  gtk_widget_add_accelerator (file_item, "activate", accel_group,
			      GDK_s, GDK_CONTROL_MASK | GDK_SHIFT_MASK,
			      GTK_ACCEL_VISIBLE);

  g_signal_connect ((gpointer) file_item, "activate",
		    G_CALLBACK (editor_save_as_m), NULL);

  image = gtk_image_new_from_stock ("gtk-save-as", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (file_item), image);

  sep = gtk_separator_menu_item_new ();
  gtk_widget_show (sep);
  gtk_container_add (GTK_CONTAINER (file_menu), sep);

  file_item = gtk_image_menu_item_new_with_mnemonic ("_Print...");
  gtk_widget_show (file_item);
  gtk_container_add (GTK_CONTAINER (file_menu), file_item);

  gtk_widget_add_accelerator (file_item, "activate", accel_group,
			      GDK_p, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  image = gtk_image_new_from_stock ("gtk-print", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (file_item), image);

  g_signal_connect ((gpointer) file_item, "activate", G_CALLBACK (print),
		    NULL);

  file_item = gtk_image_menu_item_new_with_mnemonic ("P_rint Preview...");
  gtk_widget_show (file_item);
  gtk_container_add (GTK_CONTAINER (file_menu), file_item);

  gtk_widget_add_accelerator (file_item, "activate", accel_group,
			      GDK_p, GDK_CONTROL_MASK | GDK_SHIFT_MASK,
			      GTK_ACCEL_VISIBLE);

  image = gtk_image_new_from_stock ("gtk-print-preview", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (file_item), image);

  g_signal_connect ((gpointer) file_item, "activate",
		    G_CALLBACK (print_preview), NULL);

  sep = gtk_separator_menu_item_new ();
  gtk_widget_show (sep);
  gtk_container_add (GTK_CONTAINER (file_menu), sep);

  file_item = gtk_image_menu_item_new_with_mnemonic ("_Preferences...");
  gtk_widget_show (file_item);
  gtk_container_add (GTK_CONTAINER (file_menu), file_item);

  image = gtk_image_new_from_stock ("gtk-preferences", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (file_item), image);

  g_signal_connect ((gpointer) file_item, "activate",
		    G_CALLBACK (preferences_create), NULL);

  sep = gtk_separator_menu_item_new ();
  gtk_widget_show (sep);
  gtk_container_add (GTK_CONTAINER (file_menu), sep);

  file_item = gtk_image_menu_item_new_with_mnemonic ("_Quit");
  gtk_widget_show (file_item);
  gtk_container_add (GTK_CONTAINER (file_menu), file_item);

  gtk_widget_add_accelerator (file_item, "activate", accel_group,
			      GDK_q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  image = gtk_image_new_from_stock ("gtk-quit", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (file_item), image);

  g_signal_connect ((gpointer) file_item, "activate",
		    G_CALLBACK (graphic_quit), NULL);

  view = gtk_menu_item_new_with_mnemonic ("_View");
  gtk_widget_show (view);
  gtk_container_add (GTK_CONTAINER (menubar), view);

  view_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (view), view_menu);

  view_item = gtk_check_menu_item_new_with_mnemonic ("_Parser");
  gtk_widget_show (view_item);
  gtk_container_add (GTK_CONTAINER (view_menu), view_item);
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (view_item), TRUE);

  g_signal_connect ((gpointer) view_item, "activate",
		    G_CALLBACK (graphic_view_parser), view_item);

  edit = gtk_menu_item_new_with_mnemonic ("_Edit");
  gtk_widget_show (edit);
  gtk_container_add (GTK_CONTAINER (menubar), edit);

  edit_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (edit), edit_menu);

  editor_undo_item = gtk_image_menu_item_new_with_mnemonic ("_Undo");
  gtk_widget_show (editor_undo_item);
  gtk_widget_set_sensitive (editor_undo_item, FALSE);
  gtk_container_add (GTK_CONTAINER (edit_menu), editor_undo_item);

  gtk_widget_add_accelerator (editor_undo_item, "activate", accel_group,
			      GDK_z, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  g_signal_connect ((gpointer) editor_undo_item, "activate",
		    G_CALLBACK (editor_undo), NULL);

  image = gtk_image_new_from_stock ("gtk-undo", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (editor_undo_item),
				 image);

  editor_redo_item = gtk_image_menu_item_new_with_mnemonic ("_Redo");
  gtk_widget_show (editor_redo_item);
  gtk_widget_set_sensitive (editor_redo_item, FALSE);
  gtk_container_add (GTK_CONTAINER (edit_menu), editor_redo_item);

  gtk_widget_add_accelerator (editor_redo_item, "activate", accel_group,
			      GDK_z, GDK_CONTROL_MASK | GDK_SHIFT_MASK,
			      GTK_ACCEL_VISIBLE);

  g_signal_connect ((gpointer) editor_redo_item, "activate",
		    G_CALLBACK (editor_redo), NULL);

  image = gtk_image_new_from_stock ("gtk-redo", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (editor_redo_item),
				 image);

  sep = gtk_separator_menu_item_new ();
  gtk_widget_show (sep);
  gtk_container_add (GTK_CONTAINER (edit_menu), sep);

  edit_item = gtk_image_menu_item_new_with_mnemonic ("Cu_t");
  gtk_widget_show (edit_item);
  gtk_container_add (GTK_CONTAINER (edit_menu), edit_item);

  gtk_widget_add_accelerator (edit_item, "activate", accel_group,
			      GDK_x, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  g_signal_connect ((gpointer) edit_item, "activate", G_CALLBACK (editor_cut),
		    NULL);

  image = gtk_image_new_from_stock ("gtk-cut", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (edit_item), image);

  edit_item = gtk_image_menu_item_new_with_mnemonic ("_Copy");
  gtk_widget_show (edit_item);
  gtk_container_add (GTK_CONTAINER (edit_menu), edit_item);

  gtk_widget_add_accelerator (edit_item, "activate", accel_group,
			      GDK_c, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  g_signal_connect ((gpointer) edit_item, "activate",
		    G_CALLBACK (editor_copy), NULL);

  image = gtk_image_new_from_stock ("gtk-copy", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (edit_item), image);

  edit_item = gtk_image_menu_item_new_with_mnemonic ("_Paste");
  gtk_widget_show (edit_item);
  gtk_container_add (GTK_CONTAINER (edit_menu), edit_item);

  gtk_widget_add_accelerator (edit_item, "activate", accel_group,
			      GDK_v, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  image = gtk_image_new_from_stock ("gtk-paste", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (edit_item), image);

  g_signal_connect ((gpointer) edit_item, "activate",
		    G_CALLBACK (editor_paste), NULL);

  edit_item = gtk_image_menu_item_new_with_mnemonic ("_Delete");
  gtk_widget_show (edit_item);
  gtk_container_add (GTK_CONTAINER (edit_menu), edit_item);

  image = gtk_image_new_from_stock ("gtk-delete", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (edit_item), image);

  g_signal_connect ((gpointer) edit_item, "activate",
		    G_CALLBACK (editor_delete), NULL);

  sep = gtk_separator_menu_item_new ();
  gtk_widget_show (sep);
  gtk_container_add (GTK_CONTAINER (edit_menu), sep);

  edit_item = gtk_menu_item_new_with_mnemonic ("Select _All");
  gtk_widget_show (edit_item);
  gtk_container_add (GTK_CONTAINER (edit_menu), edit_item);

  gtk_widget_add_accelerator (edit_item, "activate", accel_group,
			      GDK_a, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  g_signal_connect ((gpointer) edit_item, "activate",
		    G_CALLBACK (editor_select_all), NULL);

  highlight = gtk_menu_item_new_with_mnemonic ("H_ighlight");
  gtk_widget_show (highlight);
  gtk_container_add (GTK_CONTAINER (menubar), highlight);

  highlight_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (highlight), highlight_menu);

  highlight_normal =
    gtk_radio_menu_item_new_with_mnemonic (highlight_group, "_Normal");
  gtk_widget_show (highlight_normal);
  gtk_container_add (GTK_CONTAINER (highlight_menu), highlight_normal);

  highlight_group =
    gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (highlight_normal));

  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (highlight_normal),
				  FALSE);

  g_signal_connect ((gpointer) highlight_normal, "activate",
		    G_CALLBACK (editor_set_lang), NULL);

  highlight_xml =
    gtk_radio_menu_item_new_with_mnemonic (highlight_group, "_XML");
  gtk_widget_show (highlight_xml);
  gtk_container_add (GTK_CONTAINER (highlight_menu), highlight_xml);

  highlight_group =
    gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (highlight_xml));

  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (highlight_xml), TRUE);

  g_signal_connect ((gpointer) highlight_xml, "activate",
		    G_CALLBACK (editor_set_lang), "XML");

  tools = gtk_menu_item_new_with_mnemonic ("_Tools");
  gtk_widget_show (tools);
  gtk_container_add (GTK_CONTAINER (menubar), tools);

  tools_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (tools), tools_menu);

  tools_item = gtk_menu_item_new_with_mnemonic ("_Word Count...");
  gtk_widget_show (tools_item);
  gtk_container_add (GTK_CONTAINER (tools_menu), tools_item);

  g_signal_connect ((gpointer) tools_item, "activate",
		    G_CALLBACK (tools_wordcount), NULL);

  tools_item = gtk_menu_item_new_with_mnemonic ("_Tags List...");
  gtk_widget_show (tools_item);
  gtk_container_add (GTK_CONTAINER (tools_menu), tools_item);

  g_signal_connect ((gpointer) tools_item, "activate",
		    G_CALLBACK (tools_tagslist), NULL);

  help = gtk_menu_item_new_with_mnemonic ("_Help");
  gtk_widget_show (help);
  gtk_container_add (GTK_CONTAINER (menubar), help);

  help_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (help), help_menu);

  help_item = gtk_image_menu_item_new_with_mnemonic ("_Contents...");
  gtk_widget_show (help_item);
  gtk_container_add (GTK_CONTAINER (help_menu), help_item);

  g_signal_connect ((gpointer) help_item, "activate",
		    G_CALLBACK (help_contents), NULL);

  image = gtk_image_new_from_stock ("gtk-help", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (help_item), image);

  help_item = gtk_image_menu_item_new_with_mnemonic ("_About...");
  gtk_widget_show (help_item);
  gtk_container_add (GTK_CONTAINER (help_menu), help_item);

  g_signal_connect ((gpointer) help_item, "activate",
		    G_CALLBACK (help_about), NULL);

#ifdef GTK_STOCK_ABOUT
  image = gtk_image_new_from_stock ("gtk-about", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);
#endif

  vpaned = gtk_vpaned_new ();
  gtk_widget_show (vpaned);
  gtk_container_add (GTK_CONTAINER (vbox), vpaned);

  notebook = gtk_notebook_new ();
  gtk_widget_show (notebook);
  gtk_paned_pack1 (GTK_PANED (vpaned), notebook, TRUE, FALSE);
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);

  editor = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (editor);
  gtk_container_add (GTK_CONTAINER (notebook), editor);

  box = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (box);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), editor, box);

  button = gtk_button_new ();
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);

  if (!(re = (graphic_element *) malloc (sizeof (graphic_element))))
    fatal ("Error: memory.");

  re->widget = editor;
  re->label = "Editor";
  re->box = NULL;

  g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (graphic_pop),
		    re);

  image = gtk_image_new_from_stock ("gtk-go-up", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_container_add (GTK_CONTAINER (button), image);

  label = gtk_label_new ("Editor");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);

  if (editor_notebook (editor))
    return 1;

  viewer = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (viewer);
  gtk_container_add (GTK_CONTAINER (notebook), viewer);

  box = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (box);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), viewer, box);

  button = gtk_button_new ();
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);

  if (!(re = (graphic_element *) malloc (sizeof (graphic_element))))
    fatal ("Error: memory.");

  re->widget = viewer;
  re->label = "Viewer";
  re->box = NULL;

  g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (graphic_pop),
		    re);

  image = gtk_image_new_from_stock ("gtk-go-up", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_container_add (GTK_CONTAINER (button), image);

  label = gtk_label_new ("Viewer");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);

  if (viewer_notebook (viewer))
    return 1;

  tips = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (tips);
  gtk_container_add (GTK_CONTAINER (notebook), tips);

  box = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (box);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), tips, box);

  button = gtk_button_new ();
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);

  if (!(re = (graphic_element *) malloc (sizeof (graphic_element))))
    fatal ("Error: memory.");

  re->widget = tips;
  re->label = "Tips";
  re->box = NULL;

  g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (graphic_pop),
		    re);

  image = gtk_image_new_from_stock ("gtk-go-up", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_container_add (GTK_CONTAINER (button), image);

  label = gtk_label_new ("Tips");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);

  if (tips_notebook (tips))
    return 1;

  ibox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (ibox);
  gtk_paned_pack2 (GTK_PANED (vpaned), ibox, TRUE, TRUE);

  if (parser_create (ibox))
    return 1;

  gtk_widget_show (editorwin);
  gtk_paned_set_position (GTK_PANED (vpaned),
			  (notebook->allocation.height / 10) * 7);

  gtk_window_add_accel_group (GTK_WINDOW (editorwin), accel_group);
  return 0;
}

int
graphic_quit (GtkWidget * w, gpointer dummy)
{
  if (editor_change)
    if (editor_dialog_ask_save ())
      return TRUE;
  gtk_main_quit ();

  return FALSE;
}

void
graphic_view_parser (GtkWidget * w, gpointer dummy)
{
  static int size;
  if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (dummy)))
    {
      gtk_paned_set_position (GTK_PANED (vpaned), size);
    }
  else
    {
      size = gtk_paned_get_position (GTK_PANED (vpaned));
      gtk_paned_set_position (GTK_PANED (vpaned), vpaned->allocation.height);
    }
}

void
graphic_pop (GtkWidget * w, gpointer dummy)
{
  GtkWidget *win;
  GtkWidget *page;
  graphic_element *re = (graphic_element *) dummy;

  page = gtk_widget_ref (re->widget);
  re->widget = page;

  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook),
			    gtk_notebook_page_num (GTK_NOTEBOOK (notebook),
						   re->widget));

  win = graphic_create_sep_window (re);

  gtk_widget_show (win);
}

GtkWidget *
graphic_create_sep_window (graphic_element * re)
{
  GtkWidget *window;
  GtkWidget *frame;

  char buf[1024];

  snprintf (buf, 1024, "%s %s - XML Module Editor", NAME, VERSION);

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), buf);
  gtk_widget_set_size_request (window, gdk_screen_width () * 2 / 3,
			       gdk_screen_height () * 2 / 3);

  frame = gtk_frame_new (re->label);
  gtk_widget_show (frame);
  gtk_container_add (GTK_CONTAINER (window), frame);

  gtk_container_add (GTK_CONTAINER (frame), re->widget);
  re->box = frame;

  g_signal_connect ((gpointer) window, "delete_event",
		    G_CALLBACK (graphic_sep_window_close), re);

  return window;
}

int
graphic_sep_window_close (GtkWidget * w, GdkEvent * e, gpointer dummy)
{
  GtkWidget *page;
  GtkWidget *box;
  GtkWidget *button;
  GtkWidget *image;
  GtkWidget *label;

  graphic_element *re = (graphic_element *) dummy;

  page = gtk_widget_ref ((GtkWidget *) re->widget);
  re->widget = page;

  gtk_container_remove (GTK_CONTAINER (re->box), re->widget);
  gtk_container_add (GTK_CONTAINER (notebook), page);

  box = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (box);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), page, box);

  button = gtk_button_new ();
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);

  g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (graphic_pop),
		    re);

  image = gtk_image_new_from_stock ("gtk-go-up", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_container_add (GTK_CONTAINER (button), image);

  label = gtk_label_new (re->label);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);

  return FALSE;
}

/* EOF */
