/* gInfo - Copyright (C) 2004 bakunin - Andrea Marchesini 
 *                                      <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <editor.h>
#include <editor_logo.xpm>
#include <help.h>

#define GINFO_EDITOR "ginfo-editor"

void help_about_close (GtkWidget *, GtkWidget *);
void help_start_ginfo (void);

void
help_contents (GtkWidget * w, gpointer dummy)
{
  int fd;
  struct sockaddr_un saddr;
  char buf[1024];

  if ((fd = socket (AF_UNIX, SOCK_STREAM, 0)) < 0)
    fatal ("Socket error!");

  saddr.sun_family = AF_UNIX;
  snprintf (saddr.sun_path, 108, "%s/ginfo.sock", g_get_tmp_dir ());
  if (connect (fd, (struct sockaddr *) &saddr, sizeof (saddr)) == -1)
    {
      close (fd);
      help_start_ginfo ();

      return;
    }

  snprintf(buf, 1024, "%s\n", GINFO_EDITOR);
  write (fd, buf, strlen(buf));
  close (fd);
}

void
help_about (GtkWidget * w, gpointer dummy)
{
  GtkWidget *win;
  GtkWidget *label;
  GtkWidget *frame;
  GtkWidget *box;
  GdkPixmap *pixmap;
  GdkPixmap *mask;
  GtkWidget *image;
  GtkWidget *button;

  char buf[1024];

  snprintf (buf, 1024, "%s %s - About...", NAME, VERSION);

  win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (win), buf);
  gtk_widget_realize (win);

  frame = gtk_frame_new (NAME);
  gtk_widget_show (frame);
  gtk_container_add (GTK_CONTAINER (win), frame);

  box = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (box);
  gtk_container_add (GTK_CONTAINER (frame), box);

  pixmap =
    gdk_pixmap_create_from_xpm_d (win->window, &mask, NULL, editor_logo);
  image = gtk_image_new_from_pixmap (pixmap, mask);
  gtk_widget_show (image);
  gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);

  snprintf (buf, 1024, "Version: <span weight=\"bold\">%s</span>", VERSION);
  label = gtk_label_new (buf);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);

  snprintf (buf, 1024, "Author: <span weight=\"bold\">%s</span>",
	    AUTHOR_MARKUP);
  label = gtk_label_new (buf);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);

  snprintf (buf, 1024, "License: <span weight=\"bold\">%s</span>", COPYRIGHT);
  label = gtk_label_new (buf);
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);

  button = gtk_button_new_with_mnemonic ("_Close");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (box), button, FALSE, TRUE, 0);

  g_signal_connect ((gpointer) button, "clicked",
		    G_CALLBACK (help_about_close), win);

  gtk_widget_show (win);
}

void
help_about_close (GtkWidget * w, GtkWidget * win)
{
  gtk_widget_destroy (win);
}


void
help_start_ginfo (void)
{
  char *a[4];
  pid_t pid;

  a[0] = GINFO_SW;
  a[1] = "-id";
  a[2] = GINFO_EDITOR;
  a[3] = NULL;

  if (!fork ())
    {
      if (!(pid = fork ()))
	{
	  execvp (a[0], a);
	  exit (0);
	}
      else
	exit (0);
    }
  else
    {
      wait (NULL);
    }
}

/* EOF */
