/* gInfo - Copyright (C) 2004 bakunin - Andrea Marchesini 
 *                                      <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef GINFO_H
#define GINFO_H

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <dirent.h>
#include <dlfcn.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <libxml/parser.h>
#include <curl/curl.h>
#include <curl/types.h>
#include <curl/easy.h>
#include <pthread.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <errno.h>

#include "ginfo_plugin.h"

#define AUTHOR "Andrea Marchesini <bakunin@autistici.org>"
#define COPYRIGHT "copyright GPL 2004"

#define GINFO_TMP_DIR ((char *)g_get_tmp_dir())

#define GINFO_C	0
#define GINFO_XML 1

typedef struct _doc_
{
  char *text;

  GtkWidget *(*func) (void *);
  void *arg;
  char *id;

  struct _doc_ *next;

} doc;

typedef struct _module_
{
  int type;
  void *handle;

  char *file;
  char *name;
  int genre;
  char **words;

  doc *docs;
  doc *docs_last;

  struct _module_ *next;
} module;

typedef struct _tooltip_
{

  module *module;
  char *tooltip;

  GtkWidget *(*func) (void *);
  void *arg;
  char *id;

  struct _tooltip_ *next;

} tooltip;

typedef enum
{
  INFO_TEXT,
  INFO_NEWLINE,
  INFO_SPACE,
  INFO_IMAGE,
  INFO_FONT
} module_xml_info_tag;

typedef struct
{
  PangoStyle style;
  PangoVariant variant;
  PangoWeight weight;
  PangoStretch stretch;

  double scale;
  int size;

  char *foreground;
  char *background;

  GtkJustification justify;

  int underline;

  char *face;

  int margin_left;
  int margin_right;

} module_xml_info_font;

typedef struct _module_xml_info_
{

  module_xml_info_tag type;

  char *text;
  char *image;
  char *link;

  module_xml_info_font font;

  struct _module_xml_info_ *next;
} module_xml_info;

extern module *module_head;
extern module *module_as_default;
extern char *module_next_popup;
extern tooltip *tooltip_head;
extern int tooltip_time;
extern int tooltip_change_time;
extern int tooltip_done;
extern int tooltip_request;
extern int tooltip_number;
extern GtkWidget *mainwin;
extern char *graphic_color;
extern char *graphic_image;
extern GtkWidget *graphic_frame;
extern int nobootsplash;
extern int socket_closed;

extern GtkWidget *tutor_win;

/* modules.c */
int modules_readdir (char *);
void modules_close (void);
module *modules_search (char *);
void modules_delete (module *);
void modules_graphic_init (void);
void modules_graphic_text (char *, ...);
void modules_graphic_close (void);
void modules_set_default (char *, module *);

/* modules/c.c */
int modules_c_check (char *);

/* modules/xml.c */
int modules_xml_check (char *);
int modules_xml_server (char *);
char *modules_xml_download (char *);
void modules_xml_delete (void);
int modules_xml_dir (char *);
char *modules_xml_tmp_dir (char *);
char *modules_xml_server_check_name (char *);
void modules_xml_server_check_list (char *, char *);
char *modules_xml_info_trim (char *);

/* modules/ginfo.c */
int modules_ginfo_check (xmlDocPtr, xmlNodePtr, char *);

/* modules/docbook.c */
int modules_docbook_check (xmlDocPtr, xmlNodePtr, char *);

/* main.c */
void fatal (char *, ...);
void msg (char *, ...);
void usage (int);
char *trim (char *);
char **split_string (char *);

/* graphic.c */
void graphic_init (void);

/* tooltips.c */
void tooltips_add (ginfo_tips *, int, module *);
void tooltips_check_id (void);
void tooltips_timer (void);
int tooltips_timeout (gpointer);
void tooltips_close (void);
doc *tooltips_find_id (char *);

/* config.c */
void config_read (void);
void config_write (void);

/* tutor.c */
void tutor_init (void);
void tutor_container (GtkWidget * (*)(void *), void *);
GtkWidget *tutor_container_default (void *);
void tutor_close (GtkWidget *, gpointer);

/* menu.c */
void menu_preference (GtkWidget *, gpointer);
void menu_modules (GtkWidget *, gpointer);
void menu_about (GtkWidget *, gpointer);
void menu_editor (GtkWidget *, gpointer);
void menu_quit (GtkWidget *, gpointer);
void menu_nt (GtkWidget *, gpointer);
void menu_add_module (GtkWidget *, gpointer);
void menu_add_remote (GtkWidget *, gpointer);

/* search.c */
void search_char (char *);

/* docs.c */
int docs_add (ginfo_docs *, int, module *);

/* socket.c */
void *socket_admin (void *);
int socket_timeout_check (gpointer);

#endif
